# -*- coding: utf-8 -*-
import law
import luigi
import logging
import os
import re

from functools import partial

from analysis.tasks.FastProd import FastProd

from analysis.framework import Task, SpaceSeparatedListParameter
from analysis.mixins import TarballExtractionMixin
from analysis.util import working_directory

logger = logging.getLogger(__name__)
working_directory = partial(working_directory, logger=logger)


class CopyTables(Task, TarballExtractionMixin, law.LocalWorkflow):
    force = luigi.BoolParameter(default=False)

    merge_dir = luigi.Parameter(default='grids')

    default_channels = ["LO", "R", "V", "RRa", "RRb", "RV", "VV"]
    channels = SpaceSeparatedListParameter(
        default=default_channels,
        cls=luigi.Parameter,
        choices=("LO", "R", "V", "RR", "RRa", "RRb", "RV", "VV")
    )

    def create_branch_map(self):
        """Branch data dicts. Each branch should correspond to exactly one output."""
        return FastProd.req(self).branch_map

    def workflow_requires(self):
        if self.force:
            return {"fastprod": FastProd.req(self, acceptance=0)}
        else:
            return {"fastprod": FastProd.req(self)}

    def output(self):
        out_dir = os.getenv("ANALYSIS_PATH") or os.getcwd()
        basename = FastProd(branch=self.branch).output().basename
        outfile = re.sub('.gz$', '', basename)
        outfile = re.sub('.tar$', '', outfile) + '.log'
        return law.LocalFileTarget(
            '{}/{}/{}/{}/{}'.format(
                out_dir,
                self.merge_dir, self.name,
                self.branch_data['channel'], outfile
            )
        )

    def run(self):
        # get input and output targets
        input_ = FastProd(branch=self.branch).output()
        output = self.output()

        # extract input tarball to output parent directory
        self.extract_tarball(
            input_,
            output.parent,
            formatter="tar",
            logger=logger,
            # check that expected file was extracted
            check_targets=[
                output,
            ],
        )
